
proc rgb2hsl { rgbval } {
    global plvar hue sat light

	set red [expr 0x[string range $rgbval 1 2]]
	set green [expr 0x[string range $rgbval 3 4]]
	set blue [expr 0x[string range $rgbval 5 6]]

    set MIN [min $red $green $blue]
    set MAX [max $red $green $blue]
    set light [expr {($MIN+$MAX)/2}]
    if {$MIN == $MAX} {
        set sat 0
        set hue 0
    } else {
        if {$light < 128} {
            set sat [expr {(256*($MAX-$MIN))/($MAX+$MIN)}]
        } else {
            set sat [expr {(256*($MAX-$MIN))/(512-$MAX-$MIN)}]
        }
        set d [expr {$MAX-$MIN}].0
        set rc [expr {($MAX-$red)/$d}]
        set gc [expr {($MAX-$green)/$d}]
        set bc [expr {($MAX-$blue)/$d}]
        if {$red == $MAX} {
            set hue [expr {$bc-$gc}]
        } else {
           if {$green == $MAX} {
                set hue [expr {2+$rc-$bc}]
           } else {
                set hue [expr {4+$gc-$rc}]
           }
        }
        set hue [expr {$hue*60}]
        if {$hue < 0} {set hue [expr {$hue+360}]}
    }
    set hue [format %.0f $hue]
	#puts stderr "hue , sat , light = $hue $sat $light"
	return "$hue $sat $light"
}

proc min args {
        set x [lindex $args 0]
        foreach y $args {
                if {$y < $x} {set x $y}
        }
        return $x

}

proc max args {
        set x [lindex $args 0]
        foreach y $args {
                if {$y > $x} {set x $y}
        }
        return $x

}



# Routine for initializing color map 1 in HLS space.
proc plplot_colormap {w color1 color2} {
	global plvar

	# Independent variable of control points.
	matrix i f 2 = {0., 1.}

	set hslmin [rgb2hsl $color1] 
	set hslmax [rgb2hsl $color2] 

	# Hue ranges 
	matrix h f 2 
	h 0 = [expr [lindex $hslmin 0] * 1.0]
	h 1 = [expr [lindex $hslmax 0] * 1.0]

	# saturation of color
	matrix s f 2
	s 0 = [expr [lindex $hslmin 1] / 256.0]
	s 1 = [expr [lindex $hslmax 1] / 256.0]

	# Lightness and saturation
	matrix l f 2 
	l 0 = [expr [lindex $hslmin 2] / 256.0]
	l 1 = [expr [lindex $hslmax 2] / 256.0]

	# Integer flag array is zero (no interpolation along far-side of colour figure
	matrix rev i 2 = {0, 0}

	# Number of cmap1 colours
	# Interpolate between control points to set up default cmap1.
	$w cmd plscmap1n 256
	$w cmd plscmap1l 0 2 i h l s rev
}

#
# init colors used in plplot 
#
proc init_colors {} {
	global plvar

	set plvar(pl_colors) \
        { black red yellow green \
        aquamarine pink wheat grey \
        brown blue blueviolet cyan \
        turquoise magenta salmon white }

	set plvar(pl_colors2)                 \
        {000000 FF0000 FFFF00 008000 \
         60CC93 FF69B4 F5DEB3 999999 \
         8B4513 0000FF 6600CC 00FFFF \
         00CFC0 CC009C 6F3333 FFFFFF }
}

#
# get arguments for this plplot script
#
proc plplot_get_arguments {} {
	global argc argv plvar

	if { $argc == 0  } {
   		pl_error " No command line arguments to plplot. Usage: plplot -data test.data"
		return
	}
    set count 0
    set pix -1
    while { "TRUE" } {
        set arg [lindex $argv $count]
        incr count
        if {$arg == "" } { break }

        if { $arg == "-xcdev" } {
            set plvar(xcdev) [lindex $argv $count]
            incr count

			# allow multiple output files for png
			#	if {$plvar(xcdev) == "png"} {
			#		plsfam 1 0 10000000
			#	}
            continue
        }

        if { $arg == "-log" } {
            set plvar(log) [lindex $argv $count]
            incr count
            continue
        }

        if { $arg == "-data" } {
            set plvar(data) [lindex $argv $count]
            incr count
            continue
        }

        if { $arg == "-geometry" } {
            set plvar(geometry) [lindex $argv $count]
            incr count
            continue
        }

        if { $arg == "-mar" } {
            set plvar(mar) [lindex $argv $count]
            incr count
            continue
        }

        if { $arg == "-ptype" } {
            set plvar(ptype) [lindex $argv $count]
            incr count
            continue
        }

        if { $arg == "-cnum" } {
            set plvar(cont,num) [expr 2 + [lindex $argv $count]]
            incr count
            continue
        }

        if { $arg == "-cwidth" } {
            set plvar(cont,width) [lindex $argv $count]
            incr count
            continue
        }

        if { $arg == "-ccolor" } {
            set plvar(color,cont) [lindex $argv $count]
            incr count
            continue
        }

        if { $arg == "-ccolor_min" } {
			set plvar(color,contmin) [lindex $argv $count]
            incr count
            continue
        }

        if { $arg == "-ccolor_max" } {
			set plvar(color,contmax) [lindex $argv $count]
            incr count
            continue
        }

        if { $arg == "-alt" } {
            set plvar(surf,alt) [lindex $argv $count]
            incr count
            continue
        }

        if { $arg == "-az" } {
            set plvar(surf,az) [lindex $argv $count]
            incr count
            continue
        }

        if { $arg == "-titlecolor" } {
			set plvar(color,title) [lindex $argv $count]
            incr count
            continue
        }

        if { $arg == "-graphcolor" } {
			set plvar(color,graph) [lindex $argv $count]
            incr count
            continue
        }

        if { $arg == "-axiscolor" } {
			set plvar(color,axis) [lindex $argv $count]
            incr count
            continue
        }

        if { $arg == "-bgcolor" } {
			set plvar(color,bg) [lindex $argv $count]
            incr count
            continue
        }

        if { $arg == "-xlabel" } {
            set x [lindex $argv $count]
			regsub -all "~" $x " " plvar(label,x)
            incr count
            continue
        }

        if { $arg == "-ylabel" } {
            set y [lindex $argv $count]
			regsub -all "~" $y " " plvar(label,y)
            incr count
            continue
        }

        if { $arg == "-zlabel" } {
            set z [lindex $argv $count]
			regsub -all "~" $z " " plvar(label,z)
            incr count
            continue
        }

        if { $arg == "-title" } {
            set t [lindex $argv $count]
			regsub -all "~" $t " " plvar(label,title)
            incr count
            continue
        }
        if { $arg == "-labels" } {
            set labs [lindex $argv $count]
            set x [lindex $labs 0]
            set y [lindex $labs 1]
            set z [lindex $labs 2]
            set t [lindex $labs 2]
			regsub -all "~" $x " " plvar(label,x)
			regsub -all "~" $y " " plvar(label,y)
			regsub -all "~" $z " " plvar(label,z)
			regsub -all "~" $t " " plvar(label,title)
            incr count
            continue
        }
	}
	return TRUE
}

#
# read data file 
#
proc plplot_read_data {} {
	global plvar

	set fn $plvar(data) 

	# open data file
	if { [catch {open $fn r} fileId] } {
        pl_error "Cannot open input file '$fn'."
		return
    }
	#log_msg " -> plplot: gridfit data = $fn"

	# read dimensions of x and y respectively
    while { [gets $fileId rline] >= 0 } {
        set line [string trim $rline]
        if {[string index $line 0] == $plvar(comment)} {
			continue
		}
		set items [split $line " \n\t"]
		set xpts [lindex $items 0]
		set ypts [lindex $items 1]
		break
	}

	# allocate arrays 
    if [catch {matrix plvar(zMat) f $xpts $ypts} err] {
        pl_error "Problems with matrix allocation: $err"
		return
	} 
    matrix plvar(xArr) f $xpts
    matrix plvar(yArr) f $ypts 
	set plvar(xpts) $xpts
	set plvar(ypts) $ypts

	# read x values line
    while { [gets $fileId rline] >= 0 } {
        set line [string trim $rline]
		# skip comment lines
        if {[string index $line 0] == $plvar(comment)} {
			continue
		}
		set items [split $line " \n\t"]
		set xCount 0
		foreach ai $items {
			if { $xCount >= $xpts } {
        		pl_error "X array dimension error: More than $xpts points read in."
				return
			}
			if { [string trim $ai] == "" } {
				continue
			}
			if { $xCount == 0 } {
				set plvar(xMin) $ai
				set plvar(xMax) $ai
			} else {
				if { $ai < $plvar(xMin) } {
					set xplvar(Min) $ai
				}
				if { $ai > $plvar(xMax) } {
					set plvar(xMax) $ai
				}
			}
			plvar(xArr) $xCount = $ai
			incr xCount
		}
		if { $xCount != $xpts } {
        	pl_error "X array dimension error: Less than $xpts points read in."
			return
		}
		break
	}

	# read y values line
    while { [gets $fileId rline] >= 0 } {
        set line [string trim $rline]
        if {[string index $line 0] == $plvar(comment)} {
			continue
		}
		set items [split $line " \n\t"]
		set yCount 0
		foreach ai $items {
			if { $yCount >= $ypts } {
        		pl_error "Y array dimension error: More than $ypts points read in."
				return
			}
			if { [string trim $ai] == "" } {
				continue
			}
			if { $yCount == 0 } {
				set plvar(yMin) $ai
				set plvar(yMax) $ai
			} else {
				if { $ai < $plvar(yMin) } {
					set plvar(yMin) $ai
				}
				if { $ai > $plvar(yMax) } {
					set plvar(yMax) $ai
				}
			}
			plvar(yArr) $yCount = $ai
			incr yCount
		}
		if { $yCount != $ypts } {
        	pl_error "Y array dimension error: Less than $ypts points read in."
			return
		}
		break
	}

	# read corresponding z-values 
	set yCount 0
    while { [gets $fileId rline] >= 0 } {

		# check for comment line
        set line [string trim $rline]
        if {[string index $line 0] == $plvar(comment)} {
			continue
		}

		# get each number
		set items [split $line]
		if [catch { set yi [plvar(yArr) $yCount] } err ] {
			pl_error "Problem reading z values at index y=$yCount : $err"
			return
		}

		# init zMin and zMax
		if { $yCount == 0 } {
			set plvar(zMin) [lindex $items 0]
			set plvar(zMax) $plvar(zMin)
		}

		# process this line
		set xCount 0
		foreach ai $line {

			# check for valid data
        	if {$ai == ""} {
				continue
			}

			if [catch { set xi [plvar(xArr) $xCount] } err ] {
				pl_error "Problem reading z values at index x=$xCount : $err"
				return
			}
			if [catch { plvar(zMat) $xCount $yCount = $ai } err ] {
				pl_error "Problem reading z values at index x=$xCount y=$yCount : $err"
				return
			}

			# update zMin and zMax
			if { $ai < $plvar(zMin) } {
				set plvar(zMin) $ai
			}
			if { $ai > $plvar(zMax) } {
				set plvar(zMax) $ai
			}

			incr xCount 
		}
		incr yCount 
	}
}

proc log_msg {msg} {
	global plvar

	# open data file
	if { [catch {open $plvar(log) a} fileId] } {
        pl_error "Cannot open log file '$plvar(log)'."
		return
    }
	puts $fileId " -> plplot: $msg"
	close $fileId
}

proc pl_error {msg} {
	log_msg $msg 
	set plvar(rc) 1
}

#
# draw one contour plot 
#
proc one_contour {w nlev} {
	global plvar

	# set contours so that we hilite zmin and zmax
	set offset [expr {($plvar(zMax)-$plvar(zMin))*0.001}]
	set mmin  [expr $plvar(zMin) + $offset]
	set mmax  [expr $plvar(zMax) - $offset]
	set step [expr {($mmax-$mmin)/($nlev-1)}]

	# create contours array
    matrix clev f [expr $nlev + 1]
	set lo [expr $nlev - 1]
	set hi $nlev

	clev 0 = [expr $mmin - $step]
	clev 1 = $mmin
	clev $lo = $mmax
	clev $hi = [expr $mmax + $step]
    for {set i 2} {$i < $lo} {incr i} {
		clev $i = [expr {$mmin + $i * $step - $step}]
    }

	# fill pattern (select a predefined pattern)
	$w cmd plpsty 0

	# go to next subpage
	$w cmd pladv 0

	# specify viewport (these coords allow large diagram, resize box, + labelling room)
	$w cmd plvpor 0.1 0.98 0.1 0.9

	# viewport boundaries
	$w cmd plwind $plvar(xMin) $plvar(xMax) $plvar(yMin) $plvar(yMax)

	$w cmd plshades plvar(zMat) \
		$plvar(xMin) $plvar(xMax) $plvar(yMin) $plvar(yMax) \
		clev $nlev $plvar(cix,cont) $plvar(cont,width) 0 

	# draw a (1=red) box around viewport with labelling
	$w cmd plcol0 $plvar(cix,axis) 
	$w cmd plbox "bcnsti" 0.0 0  "bcnstvi" 0.0 0

	# axis and title labels
	$w cmd plcol0 $plvar(cix,title) 
	set title2 ""
	if {$plvar(label,title) == "default"} {
		set title2 [format " contour incr=%f" $step]
	}
	$w cmd pllab $plvar(label,x) $plvar(label,y) "$plvar(label,title) $title2"
}

#
# draw one surface plot
#
proc one_surface {w alt az} {
	global plvar

    set DRAW_LINEXY 3
    set MAG_COLOR 0x04

	$w cmd pladv 0
	$w cmd plcol0 $plvar(cix,bg) 
	$w cmd plvpor 0.0 0.9 0.0 0.9
	$w cmd plwind -1.0 1.0 -1.0 1.5

	$w cmd plw3d 1.0 1.0 1.2 $plvar(xMin) $plvar(xMax) $plvar(yMin) \
		$plvar(yMax) $plvar(zMin) $plvar(zMax) $alt $az
	$w cmd plcol0 $plvar(cix,axis) 
	$w cmd plbox3 "bnstu" $plvar(label,x) 0.0 0 \
    	"bnstu" $plvar(label,y) 0.0 0 \
    	"bcdmnstuv" $plvar(label,z) 0.0 4

    # magnitude colored wireframe plot with sides
   	$w cmd plot3d plvar(xArr) plvar(yArr) plvar(zMat) \
		[expr $DRAW_LINEXY | $MAG_COLOR] 1

	# title labels
	$w cmd plcol0 $plvar(cix,title) 
	set title2 "" 
	if {$plvar(label,title) == "default"} {
		set title2 [format " Alt=%.0f, Az=%.0f" $alt $az ]
	}
	$w cmd plmtex "t" 1.0 0.5 0.5 "$plvar(label,title) $title2"
}

#
# draw surface plots at various viewpoints
#
proc plplot_surfaces {{w loopback}} {
	global plvar

	# set colormap for surface plots 
	plplot_colormap $w $plvar(color,graph) $plvar(color,graph)

	# formulate default plot title.
	if {$plvar(label,title) == "default"} {
		set plvar(label,title) "zMin = $plvar(zMin)  zMax = $plvar(zMax)"
	}

	one_surface $w $plvar(surf,alt) $plvar(surf,az)
}

#
# draw contour plots
#
proc plplot_contours {{w loopback}} {
	global plvar

	# set colormap for contours
	plplot_colormap $w $plvar(color,contmin) $plvar(color,contmax) 

	# formulate default plot title.
	if {$plvar(label,title) == "default"} {
		set plvar(label,title) "zMin = $plvar(zMin)  zMax = $plvar(zMax)"
	}

	one_contour $w $plvar(cont,num) 
}

#
# initialize plplot environment 
#
proc plplot_init { } {
	global plvar

	set plvar(rc)	0
	init_colors

	# comment char for input datafiles
	set plvar(comment) "#"

    set plvar(xcdev) "xwin"
    set plvar(ptype) "contour"
    set plvar(geometry) "700x700"

    set plvar(cont,num) 10
	set plvar(cont,width) 1

	set plvar(surf,alt)	30.0
	set plvar(surf,az)	120.0

    set plvar(color,title)	blue
    set plvar(color,graph)	yellow
    set plvar(color,axis) 	red
    set plvar(color,bg) 	white
	set plvar(color,cont)	black
	set plvar(color,contmin) yellow
	set plvar(color,contmax) red

    set plvar(label,x) "X-axis" 
    set plvar(label,y) "Y-axis" 
    set plvar(label,z) "Z-axis" 
    set plvar(label,title) "default" 

	# get arguments for this script
	plplot_get_arguments
	if { $plvar(rc) > 0 } {return}

	# check current colors and set colormap 0
	set color_items { bg graph axis title cont contmin contmax}
	set colorNum [llength $color_items] 
	matrix red i $colorNum 
	matrix green i $colorNum
	matrix blue i $colorNum
	set j 0
	foreach item $color_items {
        set plvar(color,$item) [pl_check_rgb $plvar(color,$item)]
		red $j = [expr 0x[string range $plvar(color,$item) 1 2]]
		green $j = [expr 0x[string range $plvar(color,$item) 3 4]]
		blue $j = [expr 0x[string range $plvar(color,$item) 5 6]]
		set plvar(cix,$item) $j
		incr j 
    }
	plscmap0 red green blue $colorNum
	
	# init plotting package
	plinit
}

#
# check rgb variables, handle some predefined colors. 
# return white if color cannot be understood
#
proc pl_check_rgb {c} {
    global plvar 

    # see if we have a pre-defined color setting
    set ix [lsearch $plvar(pl_colors) $c]
    if {$ix >= 0 } {
        set c [lindex $plvar(pl_colors2) $ix]
    } else {
        # ignore '#' at beginning
        if { [string range $c 0 0] == "#" } {
            set c [string range $c 1 6]
        }
        # check each hex number
        for {set i 0} {$i < 6} {incr i} {
            if [catch { set r [expr 0x[string range $c $i $i]] } err] {
                return "#FFFFFF"
            }
            if {$r < 0 || $r > 15} {
                return "#FFFFFF"
            }
        }
    }
    return #$c
}

#
# main program
#
proc plplot_main { } {
	global plvar

	plplot_init
	if { $plvar(rc) > 0 } {return}
	plplot_read_data
	if { $plvar(rc) > 0 } {return}

	if { $plvar(ptype) == "contour" } {
		plplot_contours
	}
	if { $plvar(ptype) == "surface" } {
		plplot_surfaces
	}
	plend
}

plplot_main
